//+------------------------------------------------------------------+
//|                                              DrawManyObjects.mq5 |
//|                        Copyright 2010, MetaQuotes Software Corp. |
//|                                              http://www.mql5.com |
//+------------------------------------------------------------------+
#property copyright "2010, MetaQuotes Software Corp."
#property link      "http://www.mql5.com"
#property version   "1.00"

#property script_show_inputs
input int number=100; //  
//+------------------------------------------------------------------+
//|    Max  Min            |
//+------------------------------------------------------------------+
class CChartPriceTime
  {
private:
   double            m_MaxPrice;
   double            m_MinPrice;
   datetime          m_FirstVisibleBarTime;
   datetime          m_LastVisibleBarTime;
public:
   void              Update();
   double            MaxPrice(){return(m_MaxPrice);};
   double            MinPrice(){return(m_MinPrice);};
   datetime          RightTime(){return(m_FirstVisibleBarTime);};
   datetime          LeftTime(){return(m_LastVisibleBarTime);};
   void              GetRandomPriceAndTime(double &p,datetime &t);
  };
//+------------------------------------------------------------------+
//|                                           |
//+------------------------------------------------------------------+
void CChartPriceTime::Update(void)
  {
//---      
   m_MaxPrice=ChartGetDouble(0,CHART_PRICE_MAX);
   m_MinPrice=ChartGetDouble(0,CHART_PRICE_MIN);
//---      
   int firstbar_number=(int)ChartGetInteger(0,CHART_FIRST_VISIBLE_BAR);
//---      
   int bars_in_chart=(int)ChartGetInteger(0,CHART_WIDTH_IN_BARS);
//---   
   bool indent=(firstbar_number-bars_in_chart<0);
   int indent_in_bars=(indent)?bars_in_chart-firstbar_number:0;
//---         
   datetime times[];
   int start=!indent?(firstbar_number-bars_in_chart):0;
   int copied=CopyTime(_Symbol,_Period,start,bars_in_chart,times);
   ArraySetAsSeries(times,true);
   if(copied<=0)
     {
      Print(__FUNCTION__,"    CopyTime, ",GetLastError());
      ResetLastError();
      m_FirstVisibleBarTime=-1;
      m_LastVisibleBarTime=-1;
     }
   m_FirstVisibleBarTime=times[0]+(int)(indent_in_bars)*PeriodSeconds();
   m_LastVisibleBarTime=times[bars_in_chart-indent_in_bars-1];
  }
//+------------------------------------------------------------------+
//|                              |
//+------------------------------------------------------------------+
void CChartPriceTime::GetRandomPriceAndTime(double &p,datetime &t)
  {
   int m1=MathRand();
   p=(m_MaxPrice-m_MinPrice)*m1/32767.0+m_MinPrice;
   int m2=MathRand();
   t=datetime((m_LastVisibleBarTime-m_FirstVisibleBarTime)*m2/32767)+m_FirstVisibleBarTime;
//Print(p,t,m1,m2);
  }
//+------------------------------------------------------------------+
//|                                                |
//+------------------------------------------------------------------+
class CShape
  {
private:
   int               m_type;
protected:
   CChartPriceTime  *m_chartproperties;
   color             m_color;
public:
                     CShape(){m_type=0;m_chartproperties=new CChartPriceTime;};
                    ~CShape(){delete(m_chartproperties);};
   void              SetRandomColor();
   virtual void      Draw(){m_chartproperties.Update();};
   virtual string    getTypeName(){return("Shape");};
  };
//+------------------------------------------------------------------+
//|                              |
//+------------------------------------------------------------------+
void CShape::SetRandomColor(void)
  {
//---    
//MathSrand(GetTickCount());
//---       0  10  
   int random=(MathRand()*10)/32767;
   switch(random)
     {
      case 0: m_color=Red; break;
      case 1: m_color=Blue; break;
      case 2: m_color=Green; break;
      case 3: m_color=Yellow; break;
      case 4: m_color=Brown; break;
      case 5: m_color=DeepSkyBlue; break;
      case 6: m_color=SandyBrown; break;
      case 7: m_color=Purple; break;
      case 8: m_color=OrangeRed; break;
      case 9: m_color=LemonChiffon; break;
      default: m_color=MediumAquamarine;
     }
  }
//+------------------------------------------------------------------+
//|                                                       |
//+------------------------------------------------------------------+
class CLine: public CShape
  {
private:
   int               m_type;
public:
                     CLine(){m_type=1;};
   void              Draw();
   string            getTypeName(){return("Line");};
  };
//+------------------------------------------------------------------+
//|                                                       |
//+------------------------------------------------------------------+
void  CLine::Draw(void)
  {
   m_chartproperties.Update();
   SetRandomColor();
   double p1,p2;
   datetime t1,t2;
   m_chartproperties.GetRandomPriceAndTime(p1,t1);
   m_chartproperties.GetRandomPriceAndTime(p2,t2);
   string obj_name=getTypeName()+(string)MathRand();
   bool created=ObjectCreate(0,obj_name,OBJ_TREND,0,t1,p1,t2,p2);
   if(!created) Print("   ",getTypeName(),", ",GetLastError());
//---     
   else
     {
      ObjectSetInteger(0,obj_name,OBJPROP_COLOR,m_color);
      ObjectSetInteger(0,obj_name,OBJPROP_BACK,true);
     }
  }
//+------------------------------------------------------------------+
//|                                                 |
//+------------------------------------------------------------------+
class CTriangle: public CShape
  {
private:
   int               m_type;
public:
                     CTriangle()   { m_type=2;           }
   string            getTypeName() { return("Triangle"); }
   void              Draw();
  };
//+------------------------------------------------------------------+
//|                                                 |
//+------------------------------------------------------------------+
void  CTriangle::Draw(void)
  {
   m_chartproperties.Update();
   SetRandomColor();
   double p1,p2,p3;
   datetime t1,t2,t3;
   m_chartproperties.GetRandomPriceAndTime(p1,t1);
   m_chartproperties.GetRandomPriceAndTime(p2,t2);
   m_chartproperties.GetRandomPriceAndTime(p3,t3);
   string obj_name=getTypeName()+(string)MathRand();
   bool created=ObjectCreate(0,obj_name,OBJ_TRIANGLE,0,t1,p1,t2,p2,t3,p3);
   if(!created) Print("   ",getTypeName(),", ",GetLastError());
//---     
   else
     {
      ObjectSetInteger(0,obj_name,OBJPROP_COLOR,m_color);
      ObjectSetInteger(0,obj_name,OBJPROP_BACK,true);
     }
  }
//+------------------------------------------------------------------+
//|                                               |
//+------------------------------------------------------------------+
class CRectangle: public CShape
  {
private:
   int               m_type;
public:
                     CRectangle(){m_type=3;};
   void              Draw();
   string            getTypeName(){return("Rectangle");};
  };
//+------------------------------------------------------------------+
//|                                                 |
//+------------------------------------------------------------------+
void  CRectangle::Draw(void)
  {
   m_chartproperties.Update();
   SetRandomColor();
   double p1,p2;
   datetime t1,t2;
   m_chartproperties.GetRandomPriceAndTime(p1,t1);
   m_chartproperties.GetRandomPriceAndTime(p2,t2);
   string obj_name=getTypeName()+(string)MathRand();
   bool created=ObjectCreate(0,obj_name,OBJ_RECTANGLE,0,t1,p1,t2,p2);
   if(!created) Print("   ",getTypeName(),", ",GetLastError());
//---     
   else
     {
      ObjectSetInteger(0,obj_name,OBJPROP_COLOR,m_color);
      ObjectSetInteger(0,obj_name,OBJPROP_BACK,true);
     }
  }
//+------------------------------------------------------------------+
//|                                                        |
//+------------------------------------------------------------------+
class CCircle: public CShape
  {
private:
   int               m_type;
public:
                     CCircle(){m_type=4;};
   void              Draw();
   string            getTypeName(){return("Circle");};
  };
//+------------------------------------------------------------------+
//|                                                 |
//+------------------------------------------------------------------+
void  CCircle::Draw(void)
  {
   m_chartproperties.Update();
   SetRandomColor();
   double p1,p2,p3;
   datetime t1,t2,t3;
   m_chartproperties.GetRandomPriceAndTime(p1,t1);
   m_chartproperties.GetRandomPriceAndTime(p2,t2);
   m_chartproperties.GetRandomPriceAndTime(p3,t3);
   string obj_name=getTypeName()+(string)MathRand();
   bool created=ObjectCreate(0,obj_name,OBJ_ELLIPSE,0,t1,p1,t2,p2,t3,p3);
   if(!created) Print("   ",getTypeName(),", ",GetLastError());
//---     
   else
     {
      ObjectSetInteger(0,obj_name,OBJPROP_COLOR,m_color);
      ObjectSetInteger(0,obj_name,OBJPROP_BACK,true);
     }
  }
//+------------------------------------------------------------------+
//| Script program start function                                    |
//+------------------------------------------------------------------+
void OnStart()
  {
   MathSrand(GetTickCount());
//---     CShape
   CShape *shapes[];
//---      
   ArrayResize(shapes,number);
//---     
   for(int i=0;i<number;i++)
     {
      int t=MathRand()*4/32767;
      switch(t+1)
        {
         case 1:shapes[i]=new CLine; break;
         case 2:shapes[i]=new CTriangle; break;
         case 3:shapes[i]=new CRectangle; break;
         case 4:shapes[i]=new CCircle; break;
        }
     }
//---        
   for(int i=0;i<number;i++)
     {
      //    Print(i,shapes[i].getTypeName());
      shapes[i].Draw();
      Sleep(50);
      ChartRedraw(0);
     }
//---    
   for(int i=0;i<number;i++) delete(shapes[i]);
  }
//+------------------------------------------------------------------+
